package com.gameassist.plugin;

import java.io.File;
import java.util.HashMap;
import java.util.List;

import android.app.Application;
import android.content.ComponentName;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Pair;
import android.view.View;

public abstract class Plugin {
	
	protected Context context;
	
	protected Application targetApplication;
	
	private boolean attached;
	
	private File sourcePath;
	private File libPath;
	private File sourceFile;
	private File configPath;
	
	private PackageInfo packageInfo;
	
	private PluginManager pluginManager;
	
	public Plugin() {
	}
	
	public final void attach(Context context, Application target, String sourcePath, String libPath, String sourceFile, String configPath, PackageInfo packageInfo, PluginManager pluginManager) {
		if (!attached) {
			this.context = context;
			this.attached = true;
			this.targetApplication = target;
			this.sourcePath = new File(sourcePath);
			this.libPath = new File(libPath);
			this.sourceFile = new File(sourceFile);
			this.configPath = new File(configPath);
			this.packageInfo = packageInfo;
			this.pluginManager = pluginManager;
		}
	}
	
	public final Application getTargetApplication() {
		return targetApplication;
	}
	
	public final Context getContext() {
		return context;
	}
	
	public final PluginManager getPluginManager() {
		return pluginManager;
	}
	
	public final void registerActivityCallback(ActivityCallback callback) {
		pluginManager.registerActivityCallback(this, callback);
	}
	
	public final void unregisterActivityCallback(ActivityCallback callback) {
		pluginManager.unregisterActivityCallback(this, callback);
	}
	
	public final List<Plugin> getLoadedPlugins() {
		return pluginManager.getLoadedPlugins(this);
	}
	
	public final Plugin getLoadedPlugin(String pluginName) {
		return pluginManager.getLoadedPlugin(this, pluginName);
	}
	
	public final File getSourcePath() {
		return sourcePath;
	}
	
	public final File getSourceFile() {
		return sourceFile;
	}
	
	public final File getLibraryPath() {
		return libPath;
	}
	
	public final File getConfigPath() {
		return configPath;
	}
	
	public final String getPluginName() {
		return packageInfo.packageName;
	}
	
	public final Drawable getPluginIcon() {
		return packageInfo.applicationInfo.loadIcon(targetApplication.getPackageManager());
	}
	
	public final CharSequence getPluginLabel() {
		return packageInfo.applicationInfo.loadLabel(targetApplication.getPackageManager());
	}
	
	public final PackageInfo getPluginInfo() {
		return packageInfo;
	}
	
	public final void hideSelf() {
		pluginManager.call(this, getPluginName(), 1, null);
	}
	
	public final void closeSelf() {
		pluginManager.call(this, getPluginName(), 0, null);
	}
	
	public final void enableKeypadFocus(boolean enable) {
		pluginManager.call(this, getPluginName(), enable?2:3, null);
	}
	
	public final void registerClassLoaderOverride(ClassLoader override, ClassLoaderCallback callback) {
		pluginManager.registerClassLoaderOverride(this, override, callback);
	}
	
	public HashMap<String,IBinder> getInternalServiceProxy(){
		return null;
	}
	
	public HashMap<ComponentName, IBinder> getServiceProxyByComponent(){
		return null;
	}

	public HashMap<String, IBinder> getServiceProxyByAction(){
		return null;
	}
	
	public void setHostPackageName(String packageName){
	}
	
	public abstract boolean OnPluginCreate();
	
	public abstract void OnPlguinDestroy();
	
	public abstract View OnPluginUIShow();
	
	public abstract void OnPluginUIHide();
	
	public boolean pluginHasUI() {
		return true;
	}
	
	public boolean pluginAutoHide(){
		return true;
	}
	
	public Bundle pluginCall(Plugin plugin, String pluginName, int cmd, Bundle params){
		return null;
	}
	
}